@@ -22,7 +22,7 @@ def init(path: str, name: str) -> None:
         .dockerignore
         .gitattributes
         .gitignore
-        Dockerfile
+        .buildfile
         README.md
         assets              # Where your ML models and test data live
         requirements.txt    # User-specified requirements
