@@ -3,8 +3,10 @@ from tktl.commands import BaseTaktileApiCommand, CommandBase
 from tktl.core.clients.http_client import interpret_response
 from tktl.core.config import settings
 from tktl.core.exceptions.exceptions import APIClientException
+from tktl.core.loggers import LOG
 from tktl.core.schemas.user import TaktileUser
 from tktl.login import login, logout, set_api_key
+from tktl.version import TaktileVersionChecker
 
 
 class LogInCommand(BaseTaktileApiCommand):
@@ -14,9 +16,9 @@ class LogInCommand(BaseTaktileApiCommand):
             try:
                 user = interpret_response(response, TaktileUser)
             except APIClientException as e:
-                self.logger.error(f"Authentication failed: {e.detail}")
+                LOG.error(f"Authentication failed: {e.detail}")
                 return False
-            self.logger.log(
+            LOG.log(
                 f"Authentication successful for user: {user.username}", color="green"
             )
             return True
@@ -29,13 +31,15 @@ class LogOutCommand(CommandBase):
 
 
 class ShowVersionCommand(CommandBase):
-    def execute(self):
-        self.logger.log(__version__)
+    def execute(self, check: bool = False):
+        LOG.log(__version__)
+        if check:
+            TaktileVersionChecker.look_for_new_version()
 
 
 class SetApiKeyCommand(CommandBase):
     def execute(self, api_key):
         if not api_key:
-            self.logger.error("API Key cannot be empty.")
+            LOG.error("API Key cannot be empty.")
             return
         set_api_key(api_key=api_key)
