@@ -12,15 +12,23 @@ from requests import ConnectionError, Timeout
 from tktl.core.clients import Client
 from tktl.core.clients.http_client import API, interpret_response
 from tktl.core.config import settings
-from tktl.core.exceptions.exceptions import APIClientException, EndpointException
+from tktl.core.exceptions.exceptions import (
+    APIClientException,
+    APIClientExceptionRetryable,
+)
 from tktl.core.loggers import LOG, Logger
 from tktl.core.schemas.repository import _format_http_url
 from tktl.core.t import ServiceT
 
-RETRY_STATUS_CODES = [HTTPStatus.BAD_GATEWAY, HTTPStatus.NOT_FOUND]
-# We already are sure that the endpoint exists
-# and we do the work ourselves of setting the URL
-# so the chances of it being a spurious 404 are nil
+# codes for which retry-ing is not necessary, and for which
+# precise error information can be retrieved either from
+# the request, or deployment logs
+DO_NOT_RETRY_STATUS_CODES = [
+    HTTPStatus.INTERNAL_SERVER_ERROR,
+    HTTPStatus.FORBIDDEN,
+    HTTPStatus.UNAUTHORIZED,
+    HTTPStatus.UNPROCESSABLE_ENTITY,
+]
 
 
 class RestClient(Client):
@@ -91,44 +99,57 @@ class RestClient(Client):
         @tenacity.retry(
             stop=my_stop,
             retry=tenacity.retry_if_exception_type(
-                exception_types=(Timeout, EndpointException, ConnectionError)
+                exception_types=(APIClientExceptionRetryable,)
             ),
             wait=tenacity.wait_random(min=0, max=1),
             after=my_after,
             reraise=True,
         )
         def wrapped():
-            response = self.client.post(
-                url=f"model/{self.endpoint_name}",
-                data=json.dumps(inputs),
-                timeout=timeout,
-            )
+            try:
+                response = self.client.post(
+                    url=f"model/{self.endpoint_name}",
+                    data=json.dumps(inputs),
+                    timeout=timeout,
+                )
+            except (Timeout, ConnectionError) as e:
+                raise APIClientExceptionRetryable(
+                    status_code=-1, detail="A connection error has occured"
+                ) from e
 
-            if response.status_code in RETRY_STATUS_CODES:
-                raise EndpointException
-            else:
+            if response.status_code in DO_NOT_RETRY_STATUS_CODES:
+                if response.status_code == HTTPStatus.INTERNAL_SERVER_ERROR:
+                    raise APIClientException(
+                        status_code=response.status_code,
+                        detail="An error occurred with your request. Check the deployment's logs for more information",
+                    )
+                elif response.status_code == HTTPStatus.UNPROCESSABLE_ENTITY:
+                    json_response = response.json()
+                    raise APIClientException(
+                        status_code=response.status_code,
+                        detail=f"Unprocessable request body: {json_response['detail']})",
+                    )
+                else:
+                    json_response = response.json()
+                    raise APIClientException(
+                        status_code=response.status_code,
+                        detail=f"Authentication Error: {json_response['detail']}",
+                    )
+            try:
+                response.raise_for_status()
+            except requests.exceptions.RequestException as e:
                 try:
-                    response.raise_for_status()
-                except requests.exceptions.RequestException as e:
-                    if response.status_code == HTTPStatus.UNPROCESSABLE_ENTITY:
-                        json_response = response.json()
-                        raise APIClientException(
-                            status_code=response.status_code,
-                            detail=f"Unprocessable request body: {json_response['detail']})",
-                        )
-                    else:
-                        try:
-                            json_response = response.json()
-                            raise APIClientException(
-                                status_code=e.response.status_code,
-                                detail=json_response["detail"],
-                            )
-                        except (JSONDecodeError, KeyError):
-                            content = response.text
-                            raise APIClientException(
-                                status_code=e.response.status_code,
-                                detail=f"An error occurred with your request: {content})",
-                            )
+                    json_response = response.json()
+                    raise APIClientExceptionRetryable(
+                        status_code=response.status_code,
+                        detail=json_response["detail"],
+                    ) from e
+                except (JSONDecodeError, KeyError) as e:
+                    content = response.text
+                    raise APIClientExceptionRetryable(
+                        status_code=response.status_code,
+                        detail=f"An error occurred with your request: {content})",
+                    ) from e
             return response.json()
 
         return wrapped()
